﻿
-- This code is based on PitBull EstimatedHeals so credits go to Nymbia


if not DrDamage then return end


local moduleName = "zHealEstimate" -- The 'z' is a dirty hax to make sure this addon gets used after HealthBar



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Heal Estimate"
} end)

L:RegisterTranslations("koKR", function() return {
	[moduleName] = "치유 평가"
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local BS = AceLibrary("Babble-Spell-2.2")

local watch = {
	["PRIEST"] = {
		[BS["Lesser Heal"]]       = true,
		[BS["Heal"]]              = true,
		[BS["Greater Heal"]]      = true,
		[BS["Flash Heal"]]        = true,
		[BS["Prayer of Healing"]] = true, -- TODO: Fix these 3 spells to show the heals on the other targets as well some how
		[BS["Circle of Healing"]] = true,
		[BS["Binding Heal"]]      = true
	},
	["DRUID"] = {
		[BS["Healing Touch"]] = true,
		[BS["Regrowth"]]      = true
	},
	["SHAMAN"] = {
		[BS["Lesser Healing Wave"]] = true,
		[BS["Healing Wave"]]        = true,
		[BS["Chain Heal"]]          = true
	},
	["PALADIN"] = {
		[BS["Holy Light"]]     = true,
		[BS["Flash of Light"]] = true
	}
}

do
	local class = select(2, UnitClass("player"))
	if not watch[class] then return end
	watch = watch[class] -- let's hope the rest of the watch table is garbage collected after this
end


spellamount = setmetatable({}, {
	__index = function(self, key)
		self[key] = setmetatable({}, {
			__index = function(t, k)
				-- Use the average heal amount non-crit
				local _, _, _, _, avg = DrDamage:RawNumbers(DrDamage:GetSpellInfo(key, k), key)
				t[k] = avg
				return avg
			end
		})
		return self[key]
	end
})


local target = nil
local amount



----------------------------------
--      Module Declaration      --
----------------------------------


local plugin = BunchOfBars:NewModule(moduleName, "AceEvent-2.0")

plugin.revision = tonumber(("$Revision: 39856 $"):match("%d+"))



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "UNIT_SPELLCAST_STOP")

	self:RegisterEvent("DrDamage_Update")

	self:RegisterBucketEvent({"UNIT_HEALTH", "UNIT_MAXHEALTH"}, 0.2, "UpdateUnits")
end


function plugin:OnCreate(frame)
	local bar = CreateFrame("StatusBar", nil, frame)
	bar:SetStatusBarTexture("Interface/Tooltips/UI-Tooltip-Background")
	bar:SetStatusBarColor(0, 1, 0, 0.4)
	bar:SetWidth(200)
	bar:SetHeight(18)
	bar:ClearAllPoints()
	bar:SetPoint("BOTTOMLEFT", frame.parts["HealthBar"], "BOTTOMLEFT", 0, 0)
	bar:Hide()

	return bar
end


function plugin:OnUpdate(frame, bar)
	if target and UnitName(frame.unit) == target then
		local m = UnitHealthMax(frame.unit)
		local c = UnitHealth(frame.unit)

		bar:SetMinMaxValues(0, m*2)
		bar:SetValue(c + amount)
		bar:Show()
	else
		bar:Hide()
	end
end


function plugin:UNIT_SPELLCAST_SENT(casterid, spellname, spellrank, targetname)
	if casterid ~= "player" then return end

	if watch[spellname] then
		if spellrank then
			spellrank = tonumber(spellrank:match("(%d+)$"))
		else
			spellrank = 0
		end

		local spell = spellamount[spellname]
		amount = spell[spellrank]

		if amount then
			target = targetname
			self:UpdateAll()
		end
	end
end

function plugin:UNIT_SPELLCAST_STOP(casterid)
	if casterid ~= "player" then return end

	target = nil
	self:UpdateAll()
end


function plugin:DrDamage_Update()
	for k in pairs(spellamount) do
		spellamount[k] = nil
	end
end


function plugin:UpdateUnits(units)
	if not target then return end

	for unit in pairs(units) do
        if self.core.frames[unit] then
			self:OnUpdate(self.core.frames[unit], self.core.frames[unit].parts[self.name])
		end
	end
end
